import Template from './Template.js';
import DateUtil from '@midefos/utils/src/DateUtil.js'

export default class TableColumns {

  static _defaultConfig() {
    return {
      maxWidth: 200,
      tooltip: true,
      resizable: false
    }
  }

  static _defaultTextConfig() {
    const config = TableColumns._defaultConfig();
    const textConfig = { headerFilter: true };
    return { ...config, ...textConfig };
  }

  static defaultBoolean(key, text) {
    const config = TableColumns._defaultConfig();
    const defaultBooleanConfig = { field: key, title: text, formatter: "tickCross", sorter: "boolean", hozAlign: "center" };
    return { ...config, ...defaultBooleanConfig };
  }

  // TEXT
  static text(key, text) {
    const config = TableColumns._defaultTextConfig();
    const textConfig = { title: text, field: key };
    return { ...config, ...textConfig };
  }

  static type(key, text) {
    const textColumn = TableColumns.text(key, text);
    const typeConfig = { headerFilter: 'select', headerFilterParams: { values: true } };
    return { ...textColumn, ...typeConfig };
  }

  // LINKS
  static link(textKey, URLKey, text) {
    const config = TableColumns._defaultTextConfig();
    const linkConfig = {
      title: text, field: textKey,
      formatter: "link", formatterParams: { urlField: URLKey, target: "_" }
    }
    return { ...config, ...linkConfig };
  }

  static internaLink(textKey, URL, text) {
    const config = TableColumns._defaultTextConfig();
    const internalLinkConfig = {
      title: text, field: textKey,
      formatter: "link", formatterParams: { url: URL }
    }
    return { ...config, ...internalLinkConfig };
  }

  // IMAGE
  static image(key) {
    const config = TableColumns._defaultConfig();

    const width = 75;
    const height = 75;
    const imageConfig = {
      field: key, headerSort: false,
      formatter: (cell) => {
        let image = cell.getValue();
        if (!image) image = 'logo.png';

        return `<div class="thumbnailImageContainer"
                style="height: ${height}px; width: ${width}px;">
                <img style="height: ${height}px; width: ${width}px;" src='${image}' />
                </div>
                <div class="hoverImageContainer">
                <img style="bottom: calc(${height}px + 10px); left: calc(${width}px + 10px);"
                src='${image}' />
                </div>`;
      }
    }
    return { ...config, ...imageConfig };
  }

  // NUMERIC 
  static decimal(key, text, decimals = 2) {
    const config = TableColumns._defaultTextConfig();
    const decimalConfig = {
      field: key, title: text,
      formatter: (cell) => {
        const value = cell.getValue();
        if (!value) return null;
        return value.toFixed(decimals);
      }
    }
    return { ...config, ...decimalConfig };
  }

  static price(key, text = 'Price') {
    const config = TableColumns._defaultTextConfig();
    const priceConfig = {
      title: text, field: key,
      formatter: (cell) => {
        const item = cell.getRow().getData();
        return TableColumns._buildPrice(item, key);
      }
    }
    return { ...config, ...priceConfig };
  }

  static _buildPrice(item, key) {
    let text = `<div>${item[key]}€`;
    text += '</div>';
    return text;
  }

  static _calculateFee(price, fee) {
    price *= fee;
    return Math.round(price * 100) / 100;
  }

  // PERFORMANCE
  static nanos(key, text) {
    const config = TableColumns._defaultTextConfig();
    const performanceConfig = {
      field: key, title: text,
      formatter: (cell) => {
        const nanos = cell.getValue();
        if (!nanos) return null;

        if (nanos < 1000) {
          return nanos.toFixed(2) + ' ns';
        }

        const microseconds = nanos / 1000;
        if (microseconds < 1000) {
          return microseconds.toFixed(2) + ' µs';
        }

        const milliseconds = microseconds / 1000;
        if (milliseconds < 1000) {
          return milliseconds.toFixed(2) + ' ms';
        }

        const seconds = milliseconds / 1000;
        if (seconds < 60) {
          return seconds.toFixed(2) + ' secs';
        }

        const minutes = seconds / 60;
        if (minutes < 60) {
          return minutes.toFixed(2) + ' mins';
        }

        return (minutes / 60).toFixed(2) + ' hrs';
      }
    }
    return { ...config, ...performanceConfig };
  }

  // DATE
  static unix_timestamp(key, text) {
    const config = TableColumns._defaultTextConfig();
    const datetimeConfig = {
      field: key, title: text,
      formatter: (cell) => {
        const value = cell.getValue();
        if (!value) return null;
        return new Date(value * 1000).toLocaleString();
      }
    }
    return { ...config, ...datetimeConfig };
  }

  static datetime(key, text) {
    const config = TableColumns._defaultTextConfig();
    const datetimeConfig = {
      field: key, title: text,
      formatter: (cell) => {
        const value = cell.getValue();
        if (!value) return null;
        return new Date(value).toLocaleString();
      }
    }
    return { ...config, ...datetimeConfig };
  }

  static date(key, text) {
    const config = TableColumns._defaultTextConfig();
    const dateConfig = {
      field: key, title: text,
      formatter: (cell) => {
        const value = cell.getValue();
        if (!value) return null;
        return new Date(value).toLocaleDateString();
      }
    }
    return { ...config, ...dateConfig };
  }

  static time(key, text) {
    const config = TableColumns._defaultTextConfig();
    const timeConfig = {
      field: key, title: text,
      formatter: (cell) => {
        const value = cell.getValue();
        if (!value) return null;
        return new Date(value).toLocaleTimeString();
      }
    }
    return { ...config, ...timeConfig };
  }

  static dateDiff(startKey, endKey, text) {
    const config = TableColumns._defaultTextConfig();
    const dateDiffConfig = {
      title: text,
      formatter: (cell) => {
        const item = cell.getRow().getData();
        return DateUtil.millisecondsToDHMS(DateUtil.difference(new Date(item[endKey]), new Date(item[startKey])));
      }
    }
    return { ...config, ...dateDiffConfig };
  }

  static quantity(key, text, formater = null, sorter = null) {
    const config = TableColumns._defaultTextConfig();
    if (!formater) {
      formater = (cell) => {
        const item = cell.getRow().getData();
        const items = item[key];
        return items?.length || 0;
      }
      sorter = (a, b) => a.length - b.length;
    }
    const quantityConfig = {
      field: key, title: text,
      formatter: formater, sorter: sorter
    }
    return { ...config, ...quantityConfig };
  }

  // ICONS
  static flag(action, text = '') {
    return this.icon(Template.flagIcon(), action, text)
  }

  static play(action, text = '') {
    return this.icon(Template.playIcon(), action, text)
  }

  static edit(action, text = '') {
    return this.icon(Template.editIcon(), action, text)
  }

  static trash(action, text = '') {
    return this.icon(Template.trashIcon(), action, text)
  }

  static icon(htmlIcon, action, text = '') {
    return this._iconConfig(htmlIcon, action, text)
  }

  static defaultBoolean(key, text) {
    const config = TableColumns._defaultConfig();
    const defaultBooleanConfig = { field: key, title: text, formatter: "tickCross", sorter: "boolean", hozAlign: "center" };
    return { ...config, ...defaultBooleanConfig };
  }

  static _iconConfig(icon, action, text = '') {
    const config = TableColumns._defaultConfig();
    const iconConfig = {
      title: text, headerSort: false,
      formatter: () => icon,
      cellClick: async (event, cell) => {
        await action(cell.getRow().getData(), event);
      }
    }
    return { ...config, ...iconConfig };
  }

  // CUSTOM
  static hidden(key) {
    const config = TableColumns._defaultConfig();
    const hiddenConfig = { field: key, visible: false };
    return { ...config, ...hiddenConfig };
  }

}
